# Mail service

## What it does

This service allow to deliver mails to any email address from the address specified in the configuration. It also provides two particular endpoints, one for the user to give a feedback and another one for general contact purpose. Those endpoints will both send an email to the admin address specified in the configuration.

## How it works

![mail-service](../../assets/mail-service.png)

The entrypoint of the service is a REST API provided by a [NestJS](https://github.com/nestjs/nest) application. The service builds email bodies based on the information it receives and on the provided HTML templates. It then format a JSON with all the properties (to, from, body...) expected by an SMTP server to correctly send an email.

However the service does not send this JSON directly to the distant SMTP server. Indeed as a connection failure might occure, we chose to persist this object in a RabbitMQ queue. Then a small worker written in Node.js will consume the messages from the queue and send it to the SMTP server if correctly formatted. The messages will be removed (acknoledged) from the queue only if the SMTP received the message.

## API documentation

NestJS provides a [swagger module](https://docs.nestjs.com/recipes/swagger) that can be easily integrated. Using specific annotations alongside your endpoints declaration, this module will automatically generates a swagger documentation, reachable at `/api-doc`.

## Service health

NestJS provides a [health module](https://github.com/nestjs/terminus) based on Terminus, that gives you the opportunity to declare predefined or custom health indicators. It exposes the health status of the service at `/health`.  
This service will return a `200` http status code when all indicators are healthy. Otherwise it will return a `503` http status code. 

For this service we declared an health indicator that verify that the connection to the RabbitMQ is available.

## Stats

We are using a Node module called [swagger-stats](http://swaggerstats.io/).  
It traces API calls, monitors API performance and usage statistics. It exposes the metrics in different formats, such as Prometheus format, so you may use Prometheus and Graphana for API monitoring and alerting.  
Those metrics are available at `/swagger-stats/metrics`.

For more information about this module, visit the [official swagger-stat page](http://swaggerstats.io/docs.html).

## Docker
It is possible to run this service using Docker containers, using the `docker-compose.yml` and `Dockerfile` files.  
For more information, refer to the project [service-email][add a link]